//=============================================================================
//	MuseScore - Embellish plugin
//
//	This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//
// Preparation: Add eight note rests in voice2 throughout. 
//				This creates segments needed by the plugin if there is no note in voice 1
//				Color the existing chord symbols red (or any color); the added chord symbols are black
//				Adjust mingap = 480 (length of a quarter note) if desired
//				Embellishments are added halfway between existing symbols if in between space > mingap
//				If no note is found at this position, the previous melody note will be used unless 
//				there is yet another note before the next chord symbol, e.g. a quarter dotted note 
//				followed an eight note will use the 'upbeat' eight note as melody note and also put
//				the embellishments there.
//				Experiment with inversions for best voice leading
//				For score in 3/4 adjust the onemeasure var
//
// Known issues:
// Remove all Staff Text: they hang the plugin & MS
// don't run plugin in note input mode: shuts down
// 
// Limitations:
// Ubiquitous Circle of Fifths progressions are not analyzed
// Naked chords Bb etc are recognized as Maj; others: see embel table, adaptable to your symbols
// or adapt your symbols as below: search: // Adapt alternate chord symbols
// 3/4 time sig needs work: halfway is not appropriate; try 2/3 of interval 
// ie beat 3 for chords on beat 1
//=============================================================================

import QtQuick 2.0
import MuseScore 3.0

MuseScore {
	version:	"0.2"
	description: "This plugin embellishes jazz lead sheet harmony"
	menuPath: "Plugins.Embellish"
	
	
	// create and return a new Note element with given pitch, tpc
	function createNote(pitch, tpc){
	    ////console.log("pitch = ", pitch, "tpc = ", tpc)
        var note = newElement(Element.NOTE);
        note.pitch = pitch;
        note.tpc = tpc;
        note.tpc1 = tpc;
        note.tpc2 = tpc;		
        return note;
    }
	function setCursorToTime(cursor, time){
		cursor.rewind(0);
		while (cursor.segment) { 
			var current_time = cursor.tick;
			if(current_time>=time){
				return true;
			}
			cursor.next();
		}
		cursor.rewind(0);
		return false;
	}
    function setCursorToTime2(cursor, time){
		cursor.rewind(0);
		while (cursor.segment) { 
			var current_time = cursor.tick;
			if(current_time==time){
				console.log("in setcursor cursor.tick: ", cursor.tick);
				return true;
			}
			cursor.next();
		}
		cursor.rewind(0);
		return false;
	}

function findindex(thishead, heads){
	    for(var i=0; i<heads.length; i++){
			if(heads[i][0] == thishead){
				return heads[i][1];
			}
		}		
        
    }
	function findhead(index, heads){
	    for(var i=0; i<heads.length; i++){
			if(heads[i][1] == index){
				return heads[i][0];
			}
		}		
        
    }
	function findheadlen(harmony){
		var str = harmony;
		var t = str.substr(1,1);
		if ((t == "b") || (t == "#")){
			return 2;
		}
		else{
			return 1;
		}
	}
	function findnewdistance(d1, d2){
		d1 = parseInt(d1);
		d2 = parseInt(d2);
		d1 = d1 + d2;
		////console.log("d1 = ", d1);
		if (d1 > 12) d1 -= 12;
		if (d1 < 1) d1 += 12;
		return d1;
	}
	function findDelta(d1, d2){
		d1 = parseInt(d1);
		d2 = parseInt(d2);
		d1 = d2 - d1;
		////console.log("d1 = ", d1);
		if (d1 >= 12) d1 -= 12;
		if (d1 < 0) d1 += 12;
		return d1;
	}
	onRun: {
        var tpc2note = ["cbb","gbb","dbb","abb","ebb","bbb","fb","cb","gb","db","ab","eb","bb","f","c","g","d","a","e","b","f#","c#","g#","d#","a#","e#","b#","f##","c##","g##","d##","a##","e##","b##"];
		var mingap = 480;
		var cursor = curScore.newCursor()
		if (typeof curScore === 'undefined') {
			//console.log("Please open a jazz lead sheet before calling execution of this function.");
			Qt.quit();
		}
        // collect the melody notes and their segment.tick
		var melnotes = new Array(512); 
		// melnotes
		// O: notename
		// 1: tick
		for(var i=0; i<512; i++) melnotes[i]=new Array();
		var mcount = 0;
        cursor.rewind(0); // beginning of score

        while (cursor.segment) {
            if (cursor.element && cursor.element.type === Element.CHORD) {
                var notes = cursor.element.notes;
                //var name;
				// only register top note
				i = notes.length - 1;
                melnotes[mcount][0] = tpc2note[notes[i].tpc];
				melnotes[mcount][1] = cursor.segment.tick;
				mcount++;
            }// end if CHORD 
            cursor.next();
            
        } // end while segment

		//for (var c = 0; c < mcount; c++){
		//	//console.log("note: ", melnotes[c][0] + " " + melnotes[c][1]);
		//}

        cursor.rewind(0); // beginning of score    
		cursor.voice = 1;

		//0: chord pair:"chordtype1 distance chordtype2"
		//1: distance to embed chord in half notes: from -5 (eg C-G) to 6 (eg C- F#)
		//2: embed chord type
		//3: acceptable melnotes relative to embed chords (i iib ii iiib iii iv etc...)
		// Note: bass of slash chords in absolute steps from root, e.g. D7/A -> 7/7 not 7/-5
		// Note: 7 and m7b5 voicings are identical: e.g. d g ab c = Dm7b5 = Bb7
		// Sometimes both of these will appear in the same embellish stack
		// This is because there is also another regular voicing of m7b5 = 7 1 3 b5, e.g. am7b5 = g a c eb
		// For slash chords use literal chord 1357, not voicings
		// 7#11 chords: e.g. F7#11 = G7b13 = f a b eb or inversions
		// or F9#11: a b eb g (not explored yet)
		// sorted for most prevalent first ie lowest on the score
var embel = [
["7 0 7b9",-5,"m7","101001000100"],
["7 0 7sus",-5,"m7","100001000000"],
["7 0 7sus",-2,"dim","000000000001"],
["7 0 dim",-1,"7","000100000000"],
["7 0 dim",4,"m7b5/6","000000001000"],
["7 0 m7",0,"7b13","111111111110"],
["7 0 m7",1,"dim","111111101101"],
["7 0 m7",6,"7","111110111111"],
["7 0 m7",1,"7","011011101101"],
["7 0 m7",-5,"m7b5","101101001010"],
["7 0 m7",3,"7","101000110110"],
["7 0 m7",5,"7#11","000010000000"],
["7 0 m7",-5,"m7b5/3","000000010000"],
["7 0 m7",0,"m7b5","010000000010"],
["7 0 m7",-1,"7#11","001100100000"],
["7 0 m7",-4,"7","000010100000"],
["7 0 m7",-5,"7b13","100000001000"],
["7 0 m7",0,"m7b5/6","001000000000"],
["7 0 m7",3,"m7/7","000000010000"],
["7 0 m7b5",0,"7b13","101000000000"],
["7 0 m7b5",1,"dim","000000101001"],
["7 0 m7b5",-5,"m7b5","101001000000"],
["7 0 m7b5",6,"7","000000100000"],
["7 0 m7b5",1,"7#11","000000001000"],
["7 0 m7b5",1,"7b13","000000001000"],
["7 1 7",-5,"m7","101111011010"],
["7 1 7",-4,"7","001010100000"],
["7 1 7",5,"7","001000000100"],
["7 1 7",3,"7","000010000000"],
["7 1 7",5,"7#11","000000100000"],
["7 -1 7",5,"7","111010110111"],
["7 -1 7",6,"7","110010110111"],
["7 -1 7",0,"7b13","111000011010"],
["7 -1 7",-1,"7sus","101001011010"],
["7 -1 7",0,"7b13/4","000000100000"],
["7 -1 7",-5,"7","010010000100"],
["7 -1 7",5,"7b13","000000001000"],
["7 -1 7",-1,"7sus/5","000000000100"],
["7 -1 7",-1,"7#11","100000000100"],
["7 -1 7",-1,"m7b5/6","000100000000"],
["7 -1 7",6,"7sus","000001000000"],
["7 1 7#11",-5,"m7","000001000100"],
["7 1 7#11",2,"7","001000000000"],
["7 -1 7b9",6,"m7b5","000101000010"],
["7 -1 7b9",6,"m7","100000000000"],
["7 -1 7sus",6,"m7","101000000010"],
["7 1 dim",0,"m7","000000010000"],
["7 1 m7",-1,"7b13","000000001010"],
["7 1 m7",3,"7","000010000000"],
["7 1 m7",4,"m7","000101000000"],
["7 1 m7",-1,"7","000000000010"],
["7 1 m7",-1,"7#11","000000000010"],
["7 -1 m7",5,"7","101110111001"],
["7 -1 m7",6,"7","101010110010"],
["7 -1 m7",3,"7","010010111101"],
["7 -1 m7",-2,"dim","010110101101"],
["7 -1 m7",-2,"7#11","000110101001"],
["7 -1 m7",0,"7#11","101010100100"],
["7 -1 m7",5,"7b13","000000001000"],
["7 -1 m7",1,"7#11","100000000000"],
["7 -1 m7",4,"m7b5","101101000100"],
["7 -1 m7",3,"7#11","000000100000"],
["7 -1 m7",6,"7b9","010000000000"],
["7 -1 m7",-1,"7#11","001000000000"],
["7 -1 m7",2,"m7b5/6","000100000000"],
["7 1 m7b5",5,"7","011010101000"],
["7 1 m7b5",4,"m7","100000000100"],
["7 -1 m7b5",-2,"7","101110110100"],
["7 -1 m7b5",2,"7#11","101110001011"],
["7 -1 m7b5",5,"7","011010111000"],
["7 -1 m7b5",-1,"7","001000010000"],
["7 -1 m7b5",-3,"dim","011100000000"],
["7 -1 m7b5",-3,"7#11","000010000000"],
["7 -1 m7b5",6,"m7b5/6","000100000000"],
["7 1 Maj",-4,"7","100010100000"],
["7 1 Maj",0,"7#11","100000000000"],
["7 1 Maj",2,"7","001000100000"],
["7 1 Maj",1,"7","000000100000"],
["7 1 Maj",-4,"7b13","000000001000"],
["7 1 Maj",6,"7","000010000010"],
["7 1 Maj",0,"7b13","100000000000"],
["7 1 Maj",5,"7","000100000000"],
["7 1 Maj",5,"7b13","001000000000"],
["7 1 Maj",-5,"m7","100000000000"],
["7 -1 Maj",0,"7b13","101010111111"],
["7 -1 Maj",6,"7","101011110110"],
["7 -1 Maj",0,"7#11","011010101010"],
["7 -1 Maj",2,"7#11","101010100100"],
["7 -1 Maj",-2,"7","011000100111"],
["7 -1 Maj",-1,"7b13","101010001010"],
["7 -1 Maj",5,"7","011010100100"],
["7 -1 Maj",2,"7","001000010000"],
["7 -1 Maj",-3,"7","101010000010"],
["7 -1 Maj",1,"dim","000100000000"],
["7 -1 Maj",1,"m7b5/6","000100000000"],
["7 -1 Maj",-2,"m7b5/6","000000100000"],
["7 -1 Maj",4,"7","001000110000"],
["7 -1 Maj",-4,"7#11","001000000000"],
["7 -1 Maj",6,"m7b5","000000001010"],
["7 -1 Maj",-5,"dim","000000000001"],
["7 -1 Maj",-5,"m7b5/6","000100000000"],
["7 -1 Maj",-4,"m7b5/6","000100000000"],
["7 -1 Maj",4,"m7b5/6","001000000000"],
["7 -1 mMaj7",5,"7","010000000000"],
["7 2 7",3,"7","000000110111"],
["7 2 7",1,"7","110001100000"],
["7 2 7",3,"7#11","000110100010"],
["7 2 7",-5,"7","000001000001"],
["7 2 7",6,"7","100000010000"],
["7 2 7",-1,"m7b5","000001000000"],
["7 2 7",1,"7b13","000000001000"],
["7 2 7",-3,"7b13","100000000000"],
["7 2 7",-3,"m7","000000000100"],
["7 2 7",4,"7#11","000100000000"],
["7 -2 7",5,"7","011010010101"],
["7 -2 7",-4,"7","010010000001"],
["7 -2 7",-5,"7","101000000000"],
["7 -2 7",6,"7","011000100000"],
["7 -2 7",4,"7#11","000100000000"],
["7 -2 7",0,"7#11","000010000000"],
["7 -2 7",3,"7","000010000000"],
["7 -2 7",-1,"7","000000010000"],
["7 -2 7",1,"7b13","000000001000"],
["7 -2 7",-1,"m7b5","000100000000"],
["7 -2 7",-2,"7#11","000000100000"],
["7 -2 7",6,"7b13","000000001000"],
["7 2 7b9",-3,"m7b5","000100000000"],
["7 -2 dim",-3,"m7","100000000000"],
["7 2 m7",1,"dim","100100100001"],
["7 2 m7",3,"7","010010100100"],
["7 2 m7",-2,"7","001000100100"],
["7 2 m7",-1,"7#11","011001001000"],
["7 2 m7",-2,"mMaj7","001000100100"],
["7 2 m7",3,"7b13","010000000000"],
["7 2 m7",-3,"7","000100000010"],
["7 2 m7",5,"7","000000010001"],
["7 2 m7",6,"7","010000010000"],
["7 -2 m7",-3,"dim","100100000000"],
["7 -2 m7",5,"7","000010110100"],
["7 -2 m7",-4,"7","010000100000"],
["7 -2 m7",-1,"7","100000000000"],
["7 -2 m7",3,"dim","000000001000"],
["7 2 m7b5",1,"7","000000000001"],
["7 2 Maj",3,"7","011010110111"],
["7 2 Maj",1,"dim","110111100000"],
["7 2 Maj",-3,"7","000000010110"],
["7 2 Maj",0,"7#11","001010100100"],
["7 2 Maj",1,"7","000010000100"],
["7 2 Maj",-3,"7b13","000000001000"],
["7 2 Maj",-4,"7","010000000100"],
["7 2 Maj",-4,"7b13","000000000001"],
["7 2 Maj",-5,"7#11","110000000000"],
["7 -2 Maj",-1,"7b13","000000101000"],
["7 -2 Maj",1,"dim","000000001100"],
["7 -2 Maj",-1,"7","000000000010"],
["7 -2 Maj",3,"mMaj7","000000010000"],
["7 -2 Maj",4,"dim","000100000000"],
["7 -2 Maj",-5,"7#11","000000000010"],
["7 -2 Maj",5,"7susb9","000001000000"],
["7 3 7",-2,"7","101000010111"],
["7 3 7",5,"7","101011010000"],
["7 3 7",6,"7","001010100000"],
["7 3 7",-5,"m7","100100000000"],
["7 3 7",-2,"m7","100000000100"],
["7 3 7",-3,"7","110000000000"],
["7 3 7",-2,"mMaj7","000000000001"],
["7 3 7",6,"7b13","000000100000"],
["7 -3 7",-2,"7","001010110111"],
["7 -3 7",3,"7","011011100100"],
["7 -3 7",5,"7","001100010100"],
["7 -3 7",4,"m7b5","100011000000"],
["7 -3 7",-4,"7","100010000000"],
["7 -3 7",6,"7","000000100010"],
["7 -3 7",0,"7b13","001000000000"],
["7 -3 7",-5,"7","001000000000"],
["7 3 7#11",4,"m7b5","100000000000"],
["7 3 7#11",5,"7","001000000000"],
["7 3 m7",0,"7b13","100000100110"],
["7 3 m7",-2,"dim","101000000100"],
["7 3 m7",6,"7","100010000000"],
["7 3 m7",5,"7","001010000000"],
["7 3 m7",1,"7","000000100000"],
["7 3 m7",1,"m7","000001000000"],
["7 -3 m7",-2,"7","101010110111"],
["7 -3 m7",3,"7","011010110101"],
["7 -3 m7",6,"7","111110100010"],
["7 -3 m7",-2,"dim","101010111101"],
["7 -3 m7",-2,"m7","101000010101"],
["7 -3 m7",6,"7b13","000000001001"],
["7 -3 m7",-2,"mMaj7","100000010101"],
["7 -3 m7",2,"m7b5/6","000000000010"],
["7 -3 m7",0,"7b13","100010000010"],
["7 -3 m7",-2,"7b13","000000001000"],
["7 -3 m7",-2,"7#11/4","000000001000"],
["7 3 m7b5",6,"7b13","100000001000"],
["7 3 m7b5",6,"7","000010000000"],
["7 -3 m7b5",-2,"7","101001110111"],
["7 -3 m7b5",6,"7","111010100111"],
["7 -3 m7b5",3,"7","000000100110"],
["7 -3 m7b5",-3,"m7","000100011000"],
["7 -3 m7b5",6,"7b13","001000101000"],
["7 -3 m7b5",-2,"m7b5","000110100000"],
["7 3 Maj",-1,"7","000000000010"],
["7 3 mMaj7",0,"m7","101001000010"],
["7 4 7",-3,"7","101010000100"],
["7 4 7",5,"7","010110000000"],
["7 4 7",-1,"7","001000010000"],
["7 4 7",2,"7","000000000001"],
["7 4 7",-5,"7","000000000001"],
["7 4 7",5,"7b13","000000001000"],
["7 -4 7",3,"7","000010000100"],
["7 -4 7",5,"7","001000000000"],
["7 -4 7",1,"7","000000100000"],
["7 -4 7",-2,"7","000000000100"],
["7 -4 7",0,"m7","000000100000"],
["7 -4 7",2,"7#11","000010000000"],
["7 -4 7",5,"7#11","000000000100"],
["7 4 7sus",-1,"m7b5","001000000010"],
["7 4 7sus",5,"7","000010000001"],
["7 4 7sus",6,"7","100000000000"],
["7 4 m7",5,"7","011010011000"],
["7 4 m7",0,"7#11","100110100010"],
["7 4 m7",-2,"7","101001110000"],
["7 4 m7",3,"7","100000010100"],
["7 4 m7",6,"7","100000000100"],
["7 4 m7",-1,"7","000000010000"],
["7 4 m7",-1,"7b13","000000100000"],
["7 4 m7",-2,"7b13","000000001000"],
["7 4 m7",4,"m7b5/6","000000000100"],
["7 4 m7",6,"7b13","000000000001"],
["7 -4 m7",6,"7","001010100010"],
["7 -4 m7",2,"7","100010000000"],
["7 -4 m7",-5,"7","000000010001"],
["7 -4 m7",3,"7","000010000000"],
["7 4 m7b5",3,"7","000010110001"],
["7 4 m7b5",-5,"m7","101001000000"],
["7 4 m7b5",0,"7b9","000000000100"],
["7 4 m7b5",1,"7","000000000001"],
["7 4 m7b5",-2,"7","000000100000"],
["7 4 m7b5",-2,"dim","000000000100"],
["7 4 m7b5",-3,"m7b5","000000000010"],
["7 4 m7b5",-5,"7b13","100000000000"],
["7 -4 m7b5",-1,"7","010000010000"],
["7 -4 m7b5",4,"7","000000000100"],
["7 4 Maj",3,"7","001010000100"],
["7 4 Maj",-1,"7#11","000000111000"],
["7 4 Maj",5,"7","011000010000"],
["7 4 Maj",-2,"7","000000010100"],
["7 4 Maj",-1,"7","000000010000"],
["7 4 Maj",-1,"7b13","000000001000"],
["7 4 Maj",-2,"7b13","000000001000"],
["7 4 Maj",-5,"m7b5","000001000000"],
["7 -4 Maj",1,"dim","000100000001"],
["7 -4 Maj",6,"7","000000100010"],
["7 -4 Maj",3,"7","000000000100"],
["7 4 mMaj7",-1,"7","000100000000"],
["7 -5 7",-4,"7","011000100001"],
["7 -5 7",6,"7","001010000000"],
["7 -5 7",3,"7","000010110000"],
["7 -5 7",5,"7","010001000100"],
["7 -5 7",-2,"7","100010000010"],
["7 -5 7",2,"7","100000000000"],
["7 -5 7",0,"7b13","000000000010"],
["7 -5 7",6,"7#11","000010000000"],
["7 5 7#11",-1,"7","000000010000"],
["7 5 7#11",6,"7#11","100000000000"],
["7 5 7sus",1,"dim","000010001101"],
["7 5 7sus",6,"7","010010100001"],
["7 5 7sus",0,"m7","100000010010"],
["7 5 7sus",-1,"7","010000000000"],
["7 5 7sus",-1,"7b13","000000001000"],
["7 5 dim",0,"7b9","100000010000"],
["7 5 dim",6,"7","010010100000"],
["7 5 dim",6,"7b13","000000001010"],
["7 5 dim",-1,"7","010000000000"],
["7 5 dim",-1,"7b9","000000001000"],
["7 -5 m7",1,"7","010010100101"],
["7 -5 m7",6,"7","010010101010"],
["7 -5 m7",0,"7b13","101011010000"],
["7 -5 m7",-1,"7","011000111000"],
["7 -5 m7",-1,"7b13","000000001000"],
["7 -5 m7",3,"7","000000100000"],
["7 -5 m7",-4,"7","000010100000"],
["7 -5 m7",-2,"7","000000010000"],
["7 -5 m7",-4,"7b13","000010000000"],
["7 -5 m7",5,"7#11","000010000000"],
["7 5 m7b5",6,"7","101010100000"],
["7 5 m7b5",-1,"7","000000000100"],
["7 5 m7b5",6,"7b13","000000001000"],
["7 -5 m7b5",1,"7","000000100111"],
["7 5 Maj",5,"7b13","000010000000"],
["7 5 Maj",6,"7","000000001000"],
["7 -5 Maj",1,"dim","010101101100"],
["7 -5 Maj",-4,"7","011010100001"],
["7 -5 Maj",2,"7","101000010000"],
["7 -5 Maj",5,"7","001010000100"],
["7 -5 Maj",-2,"7#11","100000100000"],
["7 -5 Maj",-4,"7b13","000000001000"],
["7 -5 Maj",0,"7#11","000000100000"],
["7 -5 Maj",3,"7","000000000100"],
["7 -5 Maj",-5,"m7b5/3","100000000000"],
["7 5 mMaj7",1,"7","010000100000"],
["7 5 mMaj7",6,"7","001000000000"],
["7 6 7#11",-5,"m7","000001010010"],
["7 6 7b9",-1,"7","000000100000"],
["7 6 dim",-5,"m7","100101010100"],
["7 6 m7",6,"7","111011110111"],
["7 6 m7",5,"7","111011100100"],
["7 6 m7",0,"7b13","111111111000"],
["7 6 m7",4,"dim","101111001011"],
["7 6 m7",4,"m7b5","101001001010"],
["7 6 m7",-5,"m7b5","101001000000"],
["7 6 m7",-1,"7#11","000100000010"],
["7 6 m7",-1,"7b13","000010001010"],
["7 6 m7",4,"m7b5/6","000100000000"],
["7 6 m7",-3,"7b13","000000010000"],
["7 6 m7",-5,"7","011000010000"],
["7 6 m7",3,"7","010000000001"],
["7 6 m7",5,"7b13","000000001000"],
["7 6 m7",-5,"7b13","010000001000"],
["7 6 m7",6,"7sus","100001000000"],
["7 6 m7b5",5,"7","001010010101"],
["7 6 m7b5",3,"7","000010100111"],
["7 6 m7b5",-1,"7","000000000010"],
["7 6 m7b5",-2,"7","000010000000"],
["7 6 m7b5",4,"m7b5/6","001000000000"],
["7 6 m7b5",6,"7","100000000000"],
["7 6 Maj",1,"7","100000000001"],
["7 6 Maj",-1,"7","010000000000"],
["7 6 Maj",5,"7","010000000000"],
["7 6 Maj",5,"7#11","000001000000"],
["7#11 0 7",2,"7","000010000000"],
["7#11 -1 7",-1,"7sus","000000010100"],
["7#11 -1 7",5,"7","001010000001"],
["7#11 -1 7",1,"7b13","000000001000"],
["7#11 -1 m7",6,"7","100000000010"],
["7#11 1 Maj",0,"7b13","000000001000"],
["7#11 -1 Maj",6,"7","100000000000"],
["7#11 2 Maj",1,"dim","010101100000"],
["7#11 2 Maj",3,"7","000011000001"],
["7#11 2 Maj",0,"dim","000000001000"],
["7#11 2 Maj",-2,"7","000010000100"],
["7#11 2 Maj",-3,"7#11","000000000011"],
["7#11 2 Maj",-2,"7#11","000000000010"],
["7#11 2 Maj",-2,"m7b5","000000000010"],
["7#11 2 Maj",-3,"7","000000010000"],
["7#11 2 Maj",-3,"m7b5","000000010000"],
["7#11 3 m7",5,"7","001000000000"],
["7#11 -3 m7",-1,"7","000000010000"],
["7#11 4 7",-1,"7","000000010000"],
["7#11 4 7",-5,"mMaj7","000100000000"],
["7#11 -4 m7",-3,"m7","000000000010"],
["7#11 5 m7",0,"7","100000000000"],
["7b13 5 Maj",0,"7","000001000000"],
["7b9 0 m7b5",3,"m7","000000000100"],
["7b9 5 m7",6,"7","011010110001"],
["7b9 5 m7",0,"7","110001010010"],
["7b9 5 m7",4,"dim","100100101100"],
["7b9 5 m7",0,"7b13","000010000000"],
["7sus 0 7",0,"dim","001011011100"],
["7sus 0 7",-2,"dim","001000000000"],
["7sus 2 m7",3,"7","000000000100"],
["7sus -3 m7",0,"7b9","000001000000"],
["dim 0 Maj",5,"7","000000010000"],
["dim 1 7",1,"7sus","000000000100"],
["dim 1 m7",2,"7","111010110100"],
["dim 1 m7",-4,"7","001010110100"],
["dim 1 m7",2,"7b13","100000001000"],
["dim -1 m7",-1,"7","000000010110"],
["dim -1 m7",0,"7","000000100000"],
["dim -1 m7",-2,"7","000000001000"],
["dim 3 Maj",4,"7b13","000000000001"],
["dim -3 Maj",4,"7","001011000001"],
["dim -3 Maj",2,"7#11","000010000000"],
["dim 4 7",4,"7sus","001000000000"],
["dim 6 Maj",5,"7","000000000100"],
["m7 0 7",5,"7","100110010110"],
["m7 0 7",1,"7","001010010001"],
["m7 0 7",1,"7b13","000000001000"],
["m7 0 dim",1,"7","010000000000"],
["m7 0 m7b5",3,"mMaj7","101010110101"],
["m7 0 m7b5",6,"7","000010100101"],
["m7 0 m7b5",3,"m7","101000100000"],
["m7 0 m7b5",3,"m7/7","000000000100"],
["m7 0 m7b5",5,"7","100010010000"],
["m7 0 m7b5",0,"7b13","000000000100"],
["m7 0 m7b5",-1,"7","000000100000"],
["m7 1 7",5,"7","101011010110"],
["m7 1 7",-4,"7","001010110101"],
["m7 1 7",2,"m7b5","010101001010"],
["m7 1 7",2,"m7","000100001000"],
["m7 1 7",-4,"mMaj7","000000000101"],
["m7 -1 7",5,"7","111111010110"],
["m7 -1 7",6,"7","111010110101"],
["m7 -1 7",5,"7b9","100111010110"],
["m7 -1 7",0,"m7b5","110101001010"],
["m7 -1 7",0,"m7b5/6","000100000000"],
["m7 -1 7",0,"7#11","000000101010"],
["m7 -1 7",0,"7#11/4","100100000000"],
["m7 -1 7",5,"m7b5","100001000000"],
["m7 -1 7",5,"7b13","000010001000"],
["m7 -1 7",-1,"7#11","000000100000"],
["m7 -1 7",6,"7b13","000000001000"],
["m7 -1 7",0,"7#11/10","000000001000"],
["m7 -1 7",1,"7","000000100000"],
["m7 -1 7#11",5,"7","101001010010"],
["m7 -1 7#11",6,"7","000000000001"],
["m7 -1 7b9",5,"m7","100000000000"],
["m7 1 dim",5,"7","001001010010"],
["m7 -1 dim",5,"7","101101000010"],
["m7 -1 dim",-1,"7","000010100100"],
["m7 -1 dim",-1,"7b13","000000001000"],
["m7 -1 dim",6,"7","000010000000"],
["m7 -1 dim",3,"dim","000000010000"],
["m7 -1 dim",5,"7b9","000000000010"],
["m7 1 m7",5,"7","000000010110"],
["m7 1 m7",-4,"7","000010100000"],
["m7 1 m7",-1,"7","000010000000"],
["m7 -1 m7",5,"7","101110010110"],
["m7 -1 m7",1,"7","000000000100"],
["m7 -1 m7",5,"7b13","000000001000"],
["m7 1 Maj",-1,"7","011010000000"],
["m7 1 Maj",5,"7","101000000010"],
["m7 1 Maj",-3,"7#11","000000000010"],
["m7 1 Maj",-4,"7","000000000100"],
["m7 1 Maj",5,"m7","000000000010"],
["m7 -1 Maj",5,"7","000001000010"],
["m7 -1 Maj",5,"7sus","000001000000"],
["m7 -1 Maj",0,"m7b5","000000000010"],
["m7 -1 Maj",-1,"7","000000000001"],
["m7 -1 Maj",-4,"7","000000010000"],
["m7 -1 Maj",5,"m7","000000000010"],
["m7 2 7",-3,"m7b5","010000100010"],
["m7 2 7",5,"7","001001000010"],
["m7 2 7",-4,"7","001000010000"],
["m7 2 7",3,"7","100000000000"],
["m7 -2 7",5,"7","000000000110"],
["m7 -2 7",-1,"7b13","000000001000"],
["m7 -2 7",4,"dim","000100000000"],
["m7 2 7b9",-3,"m7b5","000001000000"],
["m7 2 m7",1,"dim","011010101101"],
["m7 2 m7",5,"7","101011010010"],
["m7 2 m7",3,"mMaj7","101000010101"],
["m7 2 m7",0,"m7b5","101100000010"],
["m7 2 m7",-4,"7","010010010101"],
["m7 2 m7",0,"m7b5/6","000001000000"],
["m7 -2 m7",0,"dim","100101010000"],
["m7 -2 m7",-1,"7","010010100100"],
["m7 -2 m7",4,"7","000110001000"],
["m7 -2 m7",5,"7","001000010010"],
["m7 -2 m7",-1,"7b13","000000001000"],
["m7 -2 m7",-1,"m7","000000001100"],
["m7 -2 m7",-2,"7b13","001000000000"],
["m7 -2 m7",5,"7sus","000001000000"],
["m7 2 m7b5",5,"7","111101010110"],
["m7 2 m7b5",3,"7#11","101110010000"],
["m7 2 m7b5",-3,"m7b5","010101100110"],
["m7 2 m7b5",-4,"7","101010110101"],
["m7 2 m7b5",1,"7","011001100100"],
["m7 2 m7b5",3,"7","001010010000"],
["m7 2 m7b5",-1,"7","000010010101"],
["m7 2 m7b5",0,"m7b5","101000100010"],
["m7 2 m7b5",3,"dim","000010000000"],
["m7 2 m7b5",-4,"7#11","100000000000"],
["m7 2 m7b5",5,"m7","000001000000"],
["m7 2 m7b5",6,"7","000000000100"],
["m7 -2 Maj",3,"mMaj7","001000110101"],
["m7 -2 Maj",5,"7","100010010110"],
["m7 -2 Maj",0,"m7b5","101101000000"],
["m7 -2 Maj",-2,"dim","001000010001"],
["m7 3 7",5,"7","001000010110"],
["m7 3 7",2,"m7","110000000000"],
["m7 -3 7",4,"m7b5","000100000010"],
["m7 -3 7",5,"7","001000010100"],
["m7 -3 7",-1,"7","000000000100"],
["m7 -3 7",-4,"7","100000000000"],
["m7 3 7#11",5,"7","000000000010"],
["m7 -3 7b9",4,"m7b5","000000000001"],
["m7 3 m7",3,"mMaj7","101100010101"],
["m7 3 m7",0,"m7b5","101101100010"],
["m7 3 m7",5,"7","100011010110"],
["m7 3 m7",2,"m7","110100011010"],
["m7 3 m7",0,"mMaj7","101100010000"],
["m7 3 m7",1,"7#11","000000100000"],
["m7 3 m7",3,"m7b5","100000000000"],
["m7 -3 m7",5,"7","001000010110"],
["m7 -3 m7b5",0,"mMaj7","101101110100"],
["m7 -3 m7b5",6,"7","110110101101"],
["m7 -3 m7b5",5,"7","101011010110"],
["m7 -3 m7b5",-4,"7","001010010111"],
["m7 -3 m7b5",0,"m7b5","101001110100"],
["m7 -3 m7b5",0,"m7b5/6","000100000000"],
["m7 -3 m7b5",1,"7","010011100000"],
["m7 -3 m7b5",6,"7b13","000000001000"],
["m7 3 Maj",1,"7","000000100001"],
["m7 3 mMaj7",2,"m7","101000001010"],
["m7 3 mMaj7",3,"Maj","101010000001"],
["m7 -4 7",-3,"m7b5","000101100010"],
["m7 -4 7",5,"7","101001010110"],
["m7 -4 7",1,"7","011010100000"],
["m7 -4 7",3,"mMaj7","101000010101"],
["m7 -4 7",3,"7","100000000100"],
["m7 -4 7",3,"7#11","100010000000"],
["m7 -4 7",6,"7","000000100100"],
["m7 -4 7",0,"m7b5","101000000010"],
["m7 -4 7",-1,"7","000010000000"],
["m7 -4 7",0,"7b13","100000000010"],
["m7 -4 7",-1,"dim","000000001000"],
["m7 -4 7",6,"7b13","000000001000"],
["m7 -4 7#11",-3,"m7b5","000001100000"],
["m7 -4 7#11",5,"7","000000000110"],
["m7 4 dim",5,"7","001000010100"],
["m7 4 m7",5,"7","001000000000"],
["m7 -4 m7",-1,"7","000010000001"],
["m7 -4 m7",-2,"7sus","100000000000"],
["m7 -4 m7",5,"7","000000010010"],
["m7 -4 m7",5,"7sus","000001000000"],
["m7 4 m7b5",5,"7","100010000010"],
["m7 4 m7b5",1,"7","001000101000"],
["m7 -4 Maj",-3,"m7b5","000100101010"],
["m7 -4 Maj",5,"7","001000000010"],
["m7 -4 Maj",-4,"dim","001000000000"],
["m7 -4 mMaj7",-3,"m7b5","000000100000"],
["m7 -4 mMaj7",5,"7","000000000010"],
["m7 -5 7",5,"7","101000010110"],
["m7 -5 7",-4,"7","000000110100"],
["m7 -5 7",1,"7","011010000000"],
["m7 -5 7",2,"m7b5","100101000000"],
["m7 5 7#11",0,"7#11","100001000100"],
["m7 5 7#11",6,"7","000010100101"],
["m7 5 7#11",-1,"7b13","000000001000"],
["m7 5 7#11",5,"7","000000010000"],
["m7 5 7b13",6,"7","000000100000"],
["m7 -5 7b9",2,"m7","100101000000"],
["m7 -5 7b9",1,"7","000000000001"],
["m7 -5 7b9",-1,"dim","010000000000"],
["m7 5 7sus",6,"7","010010101101"],
["m7 5 7sus",1,"dim","011000000000"],
["m7 5 7sus",6,"7b13","000000001000"],
["m7 5 m7",5,"7","101011010110"],
["m7 5 m7",6,"7","011110100101"],
["m7 5 m7",-1,"7","010010100001"],
["m7 5 m7",0,"mMaj7","101101010000"],
["m7 5 m7",0,"7","101000010010"],
["m7 5 m7",3,"7#11","000010000000"],
["m7 5 m7",0,"dim","101100000000"],
["m7 5 m7",6,"7b13","000000001000"],
["m7 -5 m7",5,"7","001010010110"],
["m7 -5 m7",0,"mMaj7","001100010100"],
["m7 -5 m7",5,"7sus","001000010010"],
["m7 -5 m7",-4,"7","000000110000"],
["m7 -5 m7",2,"7","000000010000"],
["m7 5 m7b5",5,"7","001001000110"],
["m7 -5 m7b5",5,"7","000000110100"],
["m7 -5 m7b5",1,"7","010000000010"],
["m7 -5 Maj",5,"7","001000000000"],
["m7 6 7",1,"7","011010110101"],
["m7 6 7",5,"7","101000010110"],
["m7 6 m7",5,"7","100000000010"],
["m7b5 -1 7",5,"7b9","110000010110"],
["m7b5 -1 7",6,"7","100010110101"],
["m7b5 -1 7",1,"7","111010010000"],
["m7b5 -1 7",-4,"7","100000110100"],
["m7b5 -1 7",5,"m7b5","110000000010"],
["m7b5 -1 7",0,"7b13","000000001000"],
["m7b5 -1 7#11",5,"7","100000000000"],
["m7b5 -1 7b13",6,"7","000000001000"],
["m7b5 -1 m7",-1,"mMaj7","000000000101"],
["m7b5 -2 m7",-1,"7","000010000000"],
["m7b5 -2 m7",3,"m7b5/6","000100000000"],
["m7b5 -2 Maj",-3,"dim","001101100000"],
["m7b5 -2 Maj",5,"7sus","000000110110"],
["m7b5 -2 Maj",-1,"7","110110000000"],
["m7b5 -2 Maj",5,"7","000000110000"],
["m7b5 3 m7",-1,"Maj","000000010000"],
["m7b5 -3 Maj",-1,"7","000010000000"],
["m7b5 -4 m7",-1,"7","000010000000"],
["m7b5 -4 m7",-3,"7","000000100000"],
["m7b5 -4 m7",3,"dim","100000000000"],
["m7b5 5 7",5,"7sus","110001011010"],
["m7b5 5 7",-1,"7","011010110101"],
["m7b5 5 7",6,"7","100010110100"],
["m7b5 5 7",0,"m7","000101000010"],
["m7b5 5 7",0,"7","000000000010"],
["m7b5 5 7",0,"7#11","000000100000"],
["m7b5 5 7",-2,"7sus","100000000000"],
["m7b5 5 7#11",-1,"7","000000100000"],
["m7b5 5 7b9",6,"7","110111111101"],
["m7b5 5 7b9",5,"7","100000010110"],
["m7b5 5 7b9",-1,"7","110110110001"],
["m7b5 5 7b9",-3,"dim","001101100100"],
["m7b5 5 7sus",-1,"7","000000100000"],
["m7b5 5 m7",-1,"7","000010100100"],
["m7b5 5 m7",6,"7","000000000001"],
["m7b5 5 m7b5",-1,"7","000010010000"],
["m7b5 5 m7b5",6,"7","000000000100"],
["Maj 0 7",-5,"m7","101001010100"],
["Maj 0 7",6,"7","100000100010"],
["Maj 0 7",3,"7","000000000100"],
["Maj 0 7",-1,"7#11","010101010000"],
["Maj 0 7",-5,"m7b5","101000000100"],
["Maj 0 7",-2,"dim","001000101100"],
["Maj 0 7",4,"m7","100101010000"],
["Maj 0 7",-5,"m7b5/6","000000000100"],
["Maj 0 7",-1,"7","100000010000"],
["Maj 0 7",1,"m7b5","000001000010"],
["Maj 0 7",1,"7","000001000000"],
["Maj 0 7",-5,"7#11","000001000000"],
["Maj 0 7sus",0,"7","000010000000"],
["Maj 0 dim",0,"m7","000000000100"],
["Maj 0 dim",3,"7","010000000000"],
["Maj 0 m7",0,"dim","101000010101"],
["Maj 0 m7",-2,"dim","011000100101"],
["Maj 0 m7",0,"mMaj7","101100010101"],
["Maj 0 m7",1,"7","010100101011"],
["Maj 0 m7",5,"7#11","001000110110"],
["Maj 0 m7",3,"dim","010000000000"],
["Maj 0 m7",-5,"7","000000000100"],
["Maj 0 m7",1,"7b13","000000001000"],
["Maj 0 m7",-3,"m7","000100000000"],
["Maj 0 mMaj7",-3,"m7","101101010110"],
["Maj 1 7",-3,"m7","101100010010"],
["Maj 1 7",2,"m7","001001010100"],
["Maj 1 7",-4,"7","000010000001"],
["Maj -1 7",0,"7","100010010101"],
["Maj -1 7",6,"m7b5","010001100010"],
["Maj -1 7",-1,"7sus","100001000000"],
["Maj -1 7",0,"m7b5","100000000000"],
["Maj 1 dim",-3,"m7","100101110110"],
["Maj 1 dim",-2,"7","001010101101"],
["Maj 1 dim",4,"m7","100100000010"],
["Maj 1 dim",0,"mMaj7","100000110100"],
["Maj 1 dim",0,"m7","000000010000"],
["Maj 1 m7",5,"7","001000100111"],
["Maj 1 m7",0,"7#11","001110110000"],
["Maj 1 m7",0,"7","000010110000"],
["Maj 1 m7",4,"7/7","100000000000"],
["Maj 1 m7",4,"7","000000010010"],
["Maj -1 m7",0,"7","100010010111"],
["Maj -1 m7",-2,"7","001000100100"],
["Maj -1 m7",5,"7","011010110000"],
["Maj -1 m7",6,"m7b5","010100100010"],
["Maj -1 m7",5,"7sus","001000010000"],
["Maj -1 m7",0,"7sus","100000110010"],
["Maj -1 m7",0,"mMaj7","100000010101"],
["Maj -1 m7",-2,"7#11","001000100000"],
["Maj -1 m7",4,"7sus","100000001000"],
["Maj -1 m7",4,"7/7","100000000000"],
["Maj -1 m7",2,"7#11","001000000000"],
["Maj -1 m7",5,"m7/10","000000010000"],
["Maj -1 m7",0,"m7","100000000000"],
["Maj -1 m7",0,"7#11","000010000010"],
["Maj -1 m7",2,"7sus","000001010000"],
["Maj -1 m7",-3,"7#11","001000000000"],
["Maj 1 m7b5",-3,"m7","100100110010"],
["Maj 1 m7b5",5,"7","011010010001"],
["Maj 1 m7b5",-2,"7","001000100101"],
["Maj 1 m7b5",3,"7#11","100000000000"],
["Maj -1 m7b5",5,"7","101011110101"],
["Maj -1 m7b5",5,"Maj","101010110101"],
["Maj -1 m7b5",0,"7","000010010110"],
["Maj -1 m7b5",-2,"7","001000100000"],
["Maj 1 Maj",-4,"7b13","000000001000"],
["Maj 1 Maj",-4,"7","000010000001"],
["Maj 1 Maj",2,"7","001000000000"],
["Maj -1 Maj",5,"7","001000110001"],
["Maj -1 Maj",6,"7","100000100010"],
["Maj -1 Maj",0,"7#11","000010000000"],
["Maj -1 Maj",-2,"7#11","000000100000"],
["Maj 2 7",-3,"m7","101101110010"],
["Maj 2 7",5,"7","001010110111"],
["Maj 2 7",-2,"7","001010000100"],
["Maj 2 7",-3,"m7b5","000101000000"],
["Maj -2 7",5,"7","111010110101"],
["Maj -2 7",-3,"m7","101101011110"],
["Maj -2 7",-1,"m7b5","010101100000"],
["Maj -2 7",5,"Maj","101000010101"],
["Maj -2 7",4,"7","101000010010"],
["Maj -2 7",5,"7sus","001011000000"],
["Maj -2 7",5,"7sus/10","000000010000"],
["Maj -2 7",-3,"m7b5/6","000100000000"],
["Maj -2 7",-1,"m7","000000010000"],
["Maj 2 7#11",-3,"m7","000000000010"],
["Maj -2 7#11",-3,"m7","100000010000"],
["Maj 2 7b9",-3,"m7","000000010010"],
["Maj 2 7sus",1,"m7b5","000000100000"],
["Maj 2 7sus",-3,"m7","000000000010"],
["Maj -2 dim",0,"mMaj7","000000000100"],
["Maj 2 m7",-5,"m7b5/6","101011000111"],
["Maj 2 m7",3,"7","011110101101"],
["Maj 2 m7",1,"dim","010111101011"],
["Maj 2 m7",0,"dim","101010110101"],
["Maj 2 m7",-4,"7","010010100011"],
["Maj 2 m7",0,"7#11","100010100001"],
["Maj 2 m7",-3,"m7","100100010110"],
["Maj 2 m7",-5,"7#11","001010000101"],
["Maj 2 m7",5,"7","001010000000"],
["Maj -2 m7",0,"dim","100010010100"],
["Maj -2 m7",0,"7","100000010000"],
["Maj -2 m7",-3,"m7","000100000010"],
["Maj -2 m7",6,"7","000000100000"],
["Maj -2 m7",6,"m7b5","010000100000"],
["Maj 2 m7b5",-3,"m7","000101010010"],
["Maj 2 m7b5",3,"7","000010000101"],
["Maj 2 m7b5",5,"7","000000010101"],
["Maj 3 7",-2,"7","001011100101"],
["Maj 3 7",4,"m7b5","101101001000"],
["Maj 3 7",-3,"m7","001100010010"],
["Maj 3 7",0,"7","100110010000"],
["Maj 3 7",4,"m7","100100011010"],
["Maj 3 7",-3,"7","101100110010"],
["Maj 3 7",-2,"7#11","011001100000"],
["Maj 3 7",4,"7b13","000000001010"],
["Maj 3 7",0,"7#11","000000100000"],
["Maj 3 7",-2,"7b13","000000001000"],
["Maj 3 7",4,"7","000000010000"],
["Maj 3 7",-4,"m7b5/6","000100000000"],
["Maj -3 7",-2,"7","011010100101"],
["Maj -3 7",0,"7","100010010100"],
["Maj -3 7",5,"7","001010110100"],
["Maj -3 7",4,"m7","100101010000"],
["Maj -3 7",-2,"7b13","000000001000"],
["Maj -3 7b9",-2,"7","001000100001"],
["Maj -3 7b9",4,"m7b5","100001001000"],
["Maj -3 7b9",4,"m7","100000000000"],
["Maj 3 dim",4,"m7","101101011010"],
["Maj 3 dim",-3,"m7","101100010110"],
["Maj 3 dim",1,"7#11","000000101010"],
["Maj 3 dim",-2,"dim","001000000000"],
["Maj 3 m7",-3,"m7","001101010010"],
["Maj 3 m7",2,"m7","101001000110"],
["Maj 3 m7",4,"m7","100100010010"],
["Maj -3 m7",-2,"7","011010111111"],
["Maj -3 m7",5,"7","001010110100"],
["Maj -3 m7",3,"7","001010100101"],
["Maj -3 m7",5,"7sus","101110010101"],
["Maj -3 m7",-1,"7#11","010001001000"],
["Maj -3 m7",-4,"dim","100110101001"],
["Maj -3 m7",5,"7#11","000010100100"],
["Maj -3 m7",-5,"7sus","100001010100"],
["Maj -3 m7",0,"7sus","100010010000"],
["Maj -3 m7",-3,"dim","100101000010"],
["Maj -3 m7",4,"7","100000010000"],
["Maj -3 m7",-1,"m7b5","000001001010"],
["Maj -3 m7",-5,"7","101000000000"],
["Maj -3 m7",2,"7#11","000010000000"],
["Maj -3 m7b5",0,"m7","101000000000"],
["Maj -3 m7b5",-3,"m7","000100000000"],
["Maj 3 Maj",-2,"dim","000000000100"],
["Maj -3 Maj",-2,"7#11","000000100000"],
["Maj -3 mMaj7",4,"7","000010000000"],
["Maj 4 7",-2,"7","011010101000"],
["Maj 4 7",-1,"m7b5","000101001000"],
["Maj 4 7",5,"7","011000110100"],
["Maj -4 7",-3,"m7","001101010010"],
["Maj -4 7",-2,"7","001010100100"],
["Maj -4 7",3,"7","010010000101"],
["Maj -4 7",0,"7sus","101010010000"],
["Maj -4 7",3,"7sus","000001000000"],
["Maj 4 7#11",5,"7","001000000000"],
["Maj 4 7b9",-1,"m7b5","000001000000"],
["Maj 4 m7",-2,"7","011010110101"],
["Maj 4 m7",5,"7","111010110101"],
["Maj 4 m7",2,"m7","101001010110"],
["Maj 4 m7",-3,"m7","101100010010"],
["Maj 4 m7",-5,"7sus","101000010100"],
["Maj 4 m7",-5,"7sus/5","000001000000"],
["Maj 4 m7",2,"dim","000000010100"],
["Maj -4 m7",-3,"m7","100100000010"],
["Maj 4 m7b5",5,"7","011010110011"],
["Maj 4 m7b5",-2,"7#11","001000111101"],
["Maj 4 m7b5",-2,"7","011001010101"],
["Maj 4 m7b5",2,"m7","001101010110"],
["Maj 4 m7b5",4,"m7","101101010000"],
["Maj 4 m7b5",5,"7#11","011010110010"],
["Maj 4 m7b5",3,"7","100010100100"],
["Maj 4 m7b5",-5,"m7b5","101000001001"],
["Maj 4 m7b5",0,"7#11","100110010000"],
["Maj 4 m7b5",0,"dim","000100100000"],
["Maj 4 m7b5",2,"7b9","000000010100"],
["Maj 4 m7b5",-2,"7b13","000000001000"],
["Maj 4 m7b5",-3,"7b13","000100000000"],
["Maj 4 m7b5",-5,"7#11","000010000000"],
["Maj 4 Maj",5,"7","000000010001"],
["Maj -4 Maj",3,"7","000010000100"],
["Maj -4 Maj",-3,"m7","000000000010"],
["Maj 5 7",6,"m7b5","110101101010"],
["Maj 5 7",0,"mMaj7","101000110101"],
["Maj 5 7",0,"7","100011110101"],
["Maj 5 7",-1,"7#11","000000001000"],
["Maj 5 7",-2,"7#11","011010101000"],
["Maj -5 7",-4,"7","000010000101"],
["Maj -5 7",6,"dim","000100100010"],
["Maj -5 7",2,"m7","001001010010"],
["Maj -5 7",1,"7","000000101000"],
["Maj -5 7",1,"7b13","000000000001"],
["Maj -5 7",-4,"7b13","000000001000"],
["Maj 5 7#11",6,"m7b5","010100101000"],
["Maj 5 7#11",0,"mMaj7","101000010000"],
["Maj 5 7#11",-3,"m7","101101000010"],
["Maj -5 7b13",-3,"m7","100000000000"],
["Maj -5 7sus",-3,"m7","000100010010"],
["Maj -5 7sus",5,"7","001000010001"],
["Maj -5 7sus",5,"7#11","001000000001"],
["Maj -5 7sus",-2,"7","000000100100"],
["Maj -5 7sus",3,"7","000010000100"],
["Maj -5 7sus",-2,"7#11","001000000000"],
["Maj 5 m7",-3,"m7","001100010010"],
["Maj 5 m7",0,"7","101011010001"],
["Maj 5 m7",6,"7","010000100011"],
["Maj 5 m7",2,"m7","101001000010"],
["Maj 5 m7",4,"m7","100100011010"],
["Maj 5 m7",6,"m7b5","010001100000"],
["Maj 5 m7",6,"7#11","000000100010"],
["Maj 5 m7",1,"dim","000100100001"],
["Maj 5 m7",0,"dim","100000010000"],
["Maj 5 m7",6,"7b13","000000001000"],
["Maj -5 m7",1,"7","000110100011"],
["Maj -5 m7",6,"dim","010101101011"],
["Maj -5 m7",-4,"7","010010100101"],
["Maj -5 m7",-3,"m7","001101010010"],
["Maj -5 m7",6,"7","000100100001"],
["Maj -5 m7",0,"7b13","000010010000"],
["Maj -5 m7",2,"m7","101100000100"],
["Maj -5 m7",2,"m7b5","100000000110"],
["Maj -5 m7",-4,"7b13","000000001000"],
["Maj -5 m7",1,"7b13","000000001000"],
["Maj -5 m7b5",-2,"7","000000100110"],
["Maj -5 m7b5",0,"7b13","000010000000"],
["Maj 5 Maj",-2,"7#11","001000110101"],
["Maj 5 Maj",0,"7","100010010101"],
["Maj 5 Maj",0,"7#11","101010010100"],
["Maj 5 Maj",6,"7","000000101000"],
["Maj 5 Maj",-2,"7","011000000100"],
["Maj 5 Maj",0,"7b13","001000000000"],
["Maj -5 Maj",5,"7","011001110001"],
["Maj -5 Maj",0,"mMaj7","000000110011"],
["Maj -5 Maj",1,"dim","000101100100"],
["Maj -5 Maj",2,"7","001011001010"],
["Maj 5 mMaj7",5,"Maj","000000010000"],
["Maj 5 mMaj7",6,"m7b5","010000000000"],
["Maj 6 7",0,"7","100010010110"],
["Maj 6 7",-5,"m7","100001000100"],
["Maj 6 7",1,"m7b5","000100101100"],
["Maj 6 7",1,"7#11/10","000000101000"],
["Maj 6 7",1,"7#11","000100000100"],
["Maj 6 7",0,"7#11","100000000000"],
["Maj 6 7",0,"7#11/10","000000000100"],
["Maj 6 7",5,"7","000001000000"],
["Maj 6 7#11",1,"m7b5","000000100000"],
["Maj 6 7b9",1,"m7b5","000000100000"],
["Maj 6 dim",1,"7#11","000000000001"],
["Maj 6 dim",5,"Maj","100000010000"],
["Maj 6 m7",-3,"m7","101101010010"],
["Maj 6 m7",0,"7","100010010100"],
["Maj 6 m7",5,"7","001010110100"],
["Maj 6 m7",-3,"7","000000010010"],
["Maj 6 m7b5",-3,"m7","101101110010"],
["Maj 6 m7b5",5,"7","001010110111"],
["Maj 6 m7b5",-2,"7","001011100100"],
["Maj 6 m7b5",3,"7","000010100100"],
["Maj 6 m7b5",0,"7","101010010100"],
["Maj 6 m7b5",-5,"7","101000000100"],
["Maj 6 m7b5",-5,"m7b5","100000000100"],
["Maj 6 m7b5",-5,"7sus","000001000000"],
["Maj 6 m7b5",1,"m7b5","000000000010"],
["mMaj7 -1 7",0,"m7","101000000000"],
["mMaj7 -1 7",5,"7","000000010100"],
["mMaj7 -1 m7",5,"7","001011110100"],
["mMaj7 -1 m7",0,"m7","001000000100"],
["mMaj7 2 m7",-3,"m7b5","000000001000"],
["mMaj7 2 m7b5",-3,"m7b5","000000100010"],
["mMaj7 2 m7b5",0,"m7","000000010000"],
["mMaj7 3 7",5,"7","001000000010"],
["mMaj7 -3 7",5,"7","000010000000"],
["mMaj7 -3 m7b5",0,"m7","001001010100"],
["mMaj7 -4 7",0,"m7","000100010000"],
["mMaj7 -4 7",-3,"m7b5","000000100010"],
["mMaj7 4 m7b5",1,"7b13","000000100000"],
["mMaj7 5 7",0,"m7","101100010101"],
["mMaj7 -5 m7",0,"m7","100000000000"],
["mMaj7 -5 m7",5,"7","000000010000"],
["mMaj7 -5 Maj",5,"7","001010110000"],
["mMaj7 -5 Maj",-2,"dim","110000000111"],
["mMaj7 -5 Maj",2,"7","100001110110"],
];

var heads = [
["C",1],
["D",3],
["E",5],
["F",6],
["G",8],
["A",10],
["B",12],
["Db",2],
["Eb",4],
["Fb",5],
["Gb",7],
["Ab",9],
["Bb",11],
["Cb",12],
["B#",1],
["C#",2],
["D#",4],
["E#",6],
["F#",7],
["G#",9],
["A#",11],
];

function noteUCase(s){
	s = s.toUpperCase();
	if (s.substr(1,1) == "B"){
		s = s.substr(0,1) + "b";
	}	
	return s;
}
function soundsGood(newhead,melnote, sound){
		if (melnote == "") return 1;
		var umelnote = noteUCase(melnote);
		//console.log ("umelnote = ", umelnote)	
		var i1 = findindex(newhead, heads);
		var i2 = findindex(umelnote, heads);
		var i3 = findDelta(i1, i2);
		console.log ("i1 = ", i1, "i2 = ", i2, "i3 = ", i3)
		//console.log ("sound = ", sound)
		var r = sound.substr(i3,1);
		console.log ("SoundsGood = ", r)
		if (r == "p") return 1;
		return r;
	}
		// collect the existing chord symbols and their segment.tick
		var chordnames = new Array(512); 
		// chordnames
		// O: chordname
		// 1: tick
		// 2: head
		// 3: tail
		for(var i=0; i<128; i++) chordnames[i]=new Array();
		var count = 0;
		for (var track = 0; track < /*curScore.ntracks*/ 1; ++track) {
			var segment = curScore.firstSegment();
			while (segment) {
				var aCount = 0;
				var annotation = segment.annotations[aCount];
		
				while (annotation) {
					if (annotation.type == Element.HARMONY) {
						var harmony =  annotation;
						////console.log('harmony.text = ' + harmony.text)
						var str = harmony.text;
						var n = str.search("/");
							if (n > -1){
								////console.log('str = ' + str);
								str = str.substr(0,n);
								////console.log('str = ' + str);
							}
						chordnames[count][0] = str;
						chordnames[count][1] = segment.tick;
						var headlen = findheadlen(str);
						var hstr = str.substr(0,headlen);
						// make the root uppercase
						hstr = noteUCase(hstr);
						
						chordnames[count][2] = hstr;
						// Adapt alternate chord symbols
						if (str.length - headlen > 0) chordnames[count][3] = str.substr(headlen,(str.length - headlen))
						else chordnames[count][3] = "Maj";
						if (chordnames[count][3] == "mM7") chordnames[count][3] = "mMaj7"
						if (chordnames[count][3] == "dim7") chordnames[count][3] = "dim"
						// change alt to 7
						var n = chordnames[count][0].search("7alt");
							if (n > -1){
								var i1 = findindex(chordnames[count][2], heads);
								var i2 = findnewdistance(i1, 6);
								chordnames[count][2] = findhead(i2, heads)
								chordnames[count][3] = "7";
							}
						////console.log("head -", chordnames[count][2],"-");
						////console.log("tail -",str.substr(headlen,(str.length - headlen)),"-");
						////console.log('harmony.text = ' + harmony.text);
						////console.log('time = ' + segment.tick);
						count++;					
					}
					annotation = segment.annotations[++aCount];
				}
				segment = segment.next;
			}
		}
		// 1440 voor 3/4
		// 1920 voor 4/4
		var onemeasure = 1920;
		//add a whole note of time to the last measure
		chordnames[count][1] = chordnames[count-1][1] + onemeasure;
		
		// construct the chord pairs
		for (var c = 0; c < count-1; c++){
			var gap = chordnames[c+1][1] - chordnames[c][1];
			if (gap > mingap){
				var measure = chordnames[c][1]/onemeasure + 1;
				// NB Only head is uppercase not chordname[c][0]
				console.log("chord pair: " + chordnames[c][0] + " " + chordnames[c+1][0] + " m" + measure);
				var i1 = findindex(chordnames[c][2], heads);
				var i2 = findindex(chordnames[c+1][2],heads);
				var distance = i2 - i1;
				if (distance < -5) distance += 12;
				if (distance > 6) distance -= 12;
				
				// q = "tail1 distance tail2"
				var q = chordnames[c][3] + " " + distance + " " + chordnames[c+1][3];
				// console.log("q =", q);
				// search for q in the embel table
				for (var d = 0; d < embel.length; d++){
					// this stacks embellishments halfway between existing symbols
					if (embel[d][0] == q){
						
						var newdist = findnewdistance (i1,embel[d][1]);
						var newhead = findhead(newdist, heads);
						var newchord = newhead + embel[d][2];
						// is it a slash chord?
						// in the embel table the /base is notated as offset from the root in halftones
						var n = newchord.search("/");
						if (n > -1){
							var slashi = newchord.substr(n+1, newchord.length - (n+1));
							var slashi2 = findnewdistance (slashi,newdist);
							var shead = findhead(slashi2, heads);
							newchord = newchord.substr(0,n+1) + shead;
						}
						var newtime = chordnames[c][1] + (gap/2);
						if (onemeasure == 1440 & gap == 1440){
							newtime = chordnames[c][1] + 960;
						}
						console.log("newtime: ", newtime);
						setCursorToTime(cursor, newtime); 
						// search for newtime in melnotes
						var mfound = 0;
                        var melnote = "";
                        var melnotefound = 0;
                        for (var j = 0; j < mcount; j++ && (mfound == 0)){
                            if (melnotes[j][1] == newtime){
                                //console.log("melnote: ", melnotes[j][0]);
                                melnote = melnotes[j][0];		
                                mfound = 1;
                                melnotefound = 1;
                            }
							// if no melody note at newtime
							// see if there is another melody note before the next chord
							// if so choose it for embel insert
							// otherwise use the prior note
                            if (j > 0 && melnotes[j][1] > newtime && mfound == 0){
                                if (melnotes[j][1] < chordnames[c+1][1]){
                                    melnote = melnotes[j][0];
                                    newtime = melnotes[j][1];
									console.log("newtime: ", newtime);
                                    var result = setCursorToTime2(cursor, newtime);
									console.log("result: ", result); 
                                    console.log("cursor.tick: ", cursor.tick);
                                    console.log("melnotes[j][1] = ",melnotes[j][1]);
                                    console.log("chordnames[c+1][1] = ", chordnames[c+1][1]);
                                }
                                else {
                                    melnote = melnotes[j-1][0];
                                }
                                console.log("melnote: ", melnote);
                                mfound = 1;
                            }
                        }
						console.log("newchord: ", newchord);
						var result = soundsGood(newhead,melnote, embel[d][3]);
						// no cursor segment in tuplets
						if (result == 1 && cursor.tick != 0){
							var harmony = newElement(Element.HARMONY);
							harmony.text = newchord;
							console.log("melnote: ", melnote);
							console.log("cursor.tick: ", cursor.tick);
							cursor.add(harmony);
						}
					}
				}
			}
			
		}
		Qt.quit();
	}// end on Run
}
